/***************************************************************************
 *   Copyright (C) 2006 by Martin Strasser                                 *
 *   strasser  a t  cadmaniac  d o t  org                                  *
 *   Special thanks to Mario Boikov                                        *
 *   squeeze  a t  cadmaniac  d o t  org                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "klcpufeatures.h"

KLCPUFeatures::KLCPUFeatures( const QString & name, const QString & mmcuFlag,
                              const QString& arch,
                              int maxClockSpeedHz, int internalRAMSize,
                              int EEPROMSize, bool allowsExternalRAM,
                              int maxExternalRAMSize, int flashEEPROMSize )
{
    m_name = name;
    m_mmcuFlag = mmcuFlag;
    m_arch = arch;
    m_maxClockSpeedHz = maxClockSpeedHz;
    m_internalRAMSize = internalRAMSize;
    m_EEPROMSize = EEPROMSize;
    m_allowsExternalRAM = allowsExternalRAM;
    m_maxExternalRAMSize = maxExternalRAMSize;
    m_flashEEPROMSize = flashEEPROMSize;
}


KLCPUFeatures::~KLCPUFeatures()
{
}

KLCPUFeatures::KLCPUFeatures( )
{
    m_maxIOPins = 0;
    m_minVCC = 1.8;
    m_maxVCC = 5.5;
    m_16BitTimers = 0;
    m_8BitTimers = 1;
    m_PWM_channels = 0;
    m_RTC = 0;
    m_SPI = 0;
    m_UARTs = 1;
    m_TWI = true;
    m_ISP = true;
    m_ADChannels = 0;
    m_analogComparators = 0;
    m_brownOutDetect = true;
    m_watchdog = true;
    m_onChipOscillator = true;
    m_hardwareMultiplier = false;
    m_interrupts = 0;
    m_externalInterrupts = 0;
    m_selfProgramMemory = false;
    m_packages = "";
    
    m_name = "";
    m_mmcuFlag = "";
    m_arch = "";
    m_maxClockSpeedHz = 4000000;
    m_internalRAMSize = 32;
    m_EEPROMSize = 64;
    m_allowsExternalRAM = false;
    m_maxExternalRAMSize = 0;
    m_flashEEPROMSize = 1024;
}

KLCPUFeatures::KLCPUFeatures( const QString & name, const QString & mmcuFlag,
                              const QString & arch, int flashEEPROMSize, int EEPROMSize,
                              int internalRAMSize, int maxIOPins, int maxClockSpeedHz, int maxExtRAM,
                              double minVCC, double maxVCC, int number16BitTimers,
                              int number8BitTimers, int pwmChannels, bool rtc, bool spi,
                              int uarts, bool twi, bool isp,
                              int adChannels, bool analogComparator, bool brownOutDetect,
                              bool watchdog, bool onChipOsc, bool hwMult, int interrupts,
                              int extInt, bool selfProgMem, const QString & packages )
{
    m_name = name;
    m_arch = arch;
    m_mmcuFlag = mmcuFlag;
    m_maxClockSpeedHz = maxClockSpeedHz;
    m_internalRAMSize = internalRAMSize;
    m_EEPROMSize = EEPROMSize;
    m_maxExternalRAMSize = maxExtRAM;
    m_allowsExternalRAM = maxExtRAM>0;
    m_flashEEPROMSize = flashEEPROMSize;
    m_maxIOPins = maxIOPins;
    m_minVCC = minVCC;
    m_maxVCC = maxVCC;
    m_16BitTimers = number16BitTimers;
    m_8BitTimers = number8BitTimers;
    m_PWM_channels = pwmChannels;
    m_RTC = rtc;
    m_SPI = spi;
    m_UARTs = uarts;
    m_TWI = twi;
    m_ISP = isp;
    m_ADChannels = adChannels;
    m_analogComparators = analogComparator;
    m_brownOutDetect = brownOutDetect;
    m_watchdog = watchdog;
    m_onChipOscillator = onChipOsc;
    m_hardwareMultiplier = hwMult;
    m_interrupts = interrupts;
    m_externalInterrupts = extInt;
    m_selfProgramMemory = selfProgMem;
    m_packages = packages;
}

bool KLCPUFeatures::operator <( const KLCPUFeatures & other ) const
{
    return m_name < other.name();
}


