/***************************************************************************
 *   Copyright (C) 2006 by Martin Strasser                                 *
 *   strasser  a t  cadmaniac  d o t  org                                  *
 *   Special thanks to Mario Boikov                                        *
 *   squeeze  a t  cadmaniac  d o t  org                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "klmemoryviewwidget.h"
#include "klmemorycelllistboxitem.h"
#include <knuminput.h>
#include <qcheckbox.h>
#include <qlineedit.h>
#include <qgroupbox.h>
#include <klocale.h>


KLMemoryViewWidget::KLMemoryViewWidget(QWidget *parent, const char *name)
    :KLMemoryViewWidgetBase(parent, name)
{
    int i=0;
    m_ramEnd = 512;
    
    for (i=0; i<32; i++)
    {
        lbMemory->insertItem( new KLMemoryCellListBoxItem( i, 0,
                              QString("r%1").arg(i) ) );
    }
    for (; i<=m_ramEnd; i++)
    {
        lbMemory->insertItem( new KLMemoryCellListBoxItem( i, 0, "" ) );
    }
    // By default, don't allow a memory cell to be set.
    allowSetMemoryCell( false );
}

void KLMemoryViewWidget::slotSelectionChanged()
{
}


void KLMemoryViewWidget::slotUpdateEveryChanged( int value )
{
}


void KLMemoryViewWidget::slotUpdateEnable( bool value )
{
}


void KLMemoryViewWidget::slotBinaryCBChanged()
{
    int current = 0;
    current |= cbBit7->isChecked()?1<<7:0;
    current |= cbBit6->isChecked()?1<<6:0;
    current |= cbBit5->isChecked()?1<<5:0;
    current |= cbBit4->isChecked()?1<<4:0;
    current |= cbBit3->isChecked()?1<<3:0;
    current |= cbBit2->isChecked()?1<<2:0;
    current |= cbBit1->isChecked()?1<<1:0;
    current |= cbBit0->isChecked()?1   :0;
    
    setGUIValueTo( current );
}


void KLMemoryViewWidget::slotBinaryChanged( int value )
{
    setGUIValueTo( value );
}


void KLMemoryViewWidget::slotCharChanged( const QString& value )
{
    setGUIValueTo( value.at( 0 ).unicode() );
}


void KLMemoryViewWidget::slotDecChanged( int value )
{
    setGUIValueTo( value );
}


void KLMemoryViewWidget::slotHexChanged( int value )
{
    setGUIValueTo( value );
}


void KLMemoryViewWidget::slotNameChanged( const QString& )
{
}


void KLMemoryViewWidget::slotUpdate()
{
}


void KLMemoryViewWidget::slotSetMemoryCell()
{
    KLMemoryCellListBoxItem* item =
            dynamic_cast<KLMemoryCellListBoxItem*>( lbMemory->item( lbMemory->currentItem() ) );

    if (item)
    {
        item->setValue( kisDec->value() );
        item->setName( leName->text() );
        lbMemory->updateContents();
    }
}


void KLMemoryViewWidget::setGUIValueTo(int value)
{
    if ( kisDec->value() != value )
        kisDec->setValue( value );
    if ( kisBinary->value() != value )
        kisBinary->setValue( value );
    if ( kisHex->value() != value )
        kisHex->setValue( value );
    
    if ( cbBit7->isChecked() != ((bool) (value & 0x80)) )
        cbBit7->setChecked( value & 0x80 );
    if ( cbBit6->isChecked() != ((bool) (value & 0x40)) )
        cbBit6->setChecked( value & 0x40 );
    if ( cbBit5->isChecked() != ((bool) (value & 0x20)) )
        cbBit5->setChecked( value & 0x20 );
    if ( cbBit4->isChecked() != ((bool) (value & 0x10)) )
        cbBit4->setChecked( value & 0x10 );
    if ( cbBit3->isChecked() != ((bool) (value & 0x08)) )
        cbBit3->setChecked( value & 0x08 );
    if ( cbBit2->isChecked() != ((bool) (value & 0x04)) )
        cbBit2->setChecked( value & 0x04 );
    if ( cbBit1->isChecked() != ((bool) (value & 0x02)) )
        cbBit1->setChecked( value & 0x02 );
    if ( cbBit0->isChecked() != ((bool) (value & 0x01)) )
        cbBit0->setChecked( value & 0x01 );

    if ( leChar->text() != QChar( value ) )
        leChar->setText( QChar( value ) );
}

void KLMemoryViewWidget::slotCurrentItemChanged( QListBoxItem* item_ )
{
    KLMemoryCellListBoxItem* item =
            dynamic_cast<KLMemoryCellListBoxItem*>( item_ );

    if (item)
    {
        setGUIValueTo( item->value() );
        gbMemoryCell->setTitle( QString( i18n("Memory cell 0x%1") ).arg( item->address(), 0, 16 ) );
        leName->setText( item->name() );
    }
}

void KLMemoryViewWidget::allowSetMemoryCell( bool value )
{
    if ( value )
        gbMemoryCell->show();
    else
        gbMemoryCell->hide();
}



#include "klmemoryviewwidget.moc"
