/***************************************************************************
 *   Copyright (C) 2006 by Martin Strasser                                 *
 *   strasser  a t  cadmaniac  d o t  org                                  *
 *   Special thanks to Mario Boikov                                        *
 *   squeeze  a t  cadmaniac  d o t  org                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "klproject.h"
#include "kontrollerlab.h"
#include <klocale.h>
#include "klprogrammerinterface.h"
#include <qregexp.h>


KLProgrammerInterface::KLProgrammerInterface( KLProcessManager* processManager, KLProject* project ) :
KLProcessHandler( processManager )
{
    m_project = project;
    m_readFusesBackannotate = 0;
    m_stk500Backannotate = 0;
}


KLProgrammerInterface::~KLProgrammerInterface()
{
}

QString KLProgrammerInterface::config( const QString & key ) const
{
    if ( m_configuration.find( key ) != m_configuration.end() )
        return m_configuration[key];
    return "";
}

void KLProgrammerInterface::setConfig( const QString & key, const QString & val )
{
    m_configuration[key] = val;
}


void KLProgrammerInterface::shellProcessExitedWith( int exitVal, const QString & stdout, const QString & stderr )
{
    m_runningProc = false;
    if ( (!stdout.isEmpty()) && (!stdout.isNull()) )
        m_project->parent()->slotMessageBox( exitVal, stdout );
    if ( (!stderr.isEmpty()) && (!stderr.isNull()) )
        m_project->parent()->slotMessageBox( exitVal, stderr );
    if ( ( exitVal != 0 ) && (!m_processManager->dontAbortQueueOnError()) )
    {
        m_project->parent()->slotMessageBox( exitVal, i18n("Error(s) occurred: The exit status was %1.").arg( exitVal ) );
        if ( m_readFusesBackannotate > 0 )
        {
            m_project->backannotateFuses( );
            m_readFusesBackannotate = 0;
        }
        else if ( m_stk500Backannotate > 0 )
        {
            m_project->backannotateSTK500( );
            m_stk500Backannotate = 0;
        }
    }
    else
    {
        if ( m_readFusesBackannotate > 0 )
        {
            QMap< QString, QString > fuses = parseFuses( stdout );
            m_project->backannotateFuses( fuses );
            m_readFusesBackannotate--;
        }
        else if ( m_stk500Backannotate > 0 )
        {
            m_project->backannotateSTK500( stdout );
            m_stk500Backannotate--;
        }
    }
    if ( m_readFusesBackannotate == 0 )
        m_processManager->setDontAbortQueueOnError( false );
    if ( m_processManager->procsInQueue() <= 1 )
        m_project->setProgrammerBusy( false );
}


QString KLProgrammerInterface::validPath( const KURL & url )
{
    QString retVal = url.path();
    retVal.replace( QRegExp("([^\\\\])\\s"), "\\1\\ " );
    return retVal;
}

