/***************************************************************************
 *   Copyright (C) 2006 by Martin Strasser                                 *
 *   strasser  a t  cadmaniac  d o t  org                                  *
 *   Special thanks to Mario Boikov                                        *
 *   squeeze  a t  cadmaniac  d o t  org                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "klproject.h"
#include <klprojectmanagerwidget.h>
#include <klocale.h>
#include <kglobal.h>
#include "klprogrammeruisp.h"
#include "kontrollerlab.h"

#include <kfiledialog.h>
#include <kmessagebox.h>
#include <kapplication.h>
#include <kconfig.h>
#include <kinputdialog.h>
#include "kldocumentview.h"

#include "klcompilerinterface.h"
#include "klavrgcccompiler.h"

#define NO_RAM 0

KLProject::KLProject( KontrollerLab* parent )
{
    // The project takes care of the pointers.
    m_documents.setAutoDelete( true );
    m_parent = parent;
    // m_projectManager = 0L;
    // m_projectManagerInConfigWdg = 0L;
    m_name = i18n("Unnamed project");
    // m_sourcesNode = m_headersNode = m_notesNode = m_othersNode = 0L;
    recreateListView();
    m_programmerInterface = 0L;
    m_initialized = false;
    m_unsaved = false;
    m_programmerBusy = false;
    
    m_cpuFeatures.append( KLCPUFeatures( "AT90S1200", "at90s1200", "avr1",
                          12000000, 32, 64, false, 0, 1024 ) );
    m_cpuFeatures.append( KLCPUFeatures( "ATTiny11", "attiny11", "avr1",
         1024 , 0 , 0 , 6 , 6000000 , NO_RAM,  2.7, 5.5,
         false, 1, false, false, false, false, false, false,
         false, true, false, true, true, false, 4, 1, false,
         "SOIC (EIAJ) 8") );
    m_cpuFeatures.append( KLCPUFeatures( "ATTiny12", "attiny12", "avr1",
         1024 , 64 , 0 , 6 , 8000000 , NO_RAM,  1.8, 5.5,
         false, 1, false, false, false, false, false, true,
         false, true, true, true, true, false, 5, 1, false,
         "SOIC (EIAJ) 8") );
    m_cpuFeatures.append( KLCPUFeatures( "ATTiny15", "attiny15", "avr1",
                          1600000, 32, 64, false, 0, 1024 ) );
    m_cpuFeatures.append( KLCPUFeatures( "ATTiny28", "attiny28", "avr1",
                          4000000, 32, 0, false, 0, 2048 ) );
    m_cpuFeatures.append( KLCPUFeatures( "AT90S2313", "at90s2313", "avr2",
                          20000000, 128, 128, false, 0, 2048 ) );
    m_cpuFeatures.append( KLCPUFeatures( "AT90S2323", "at90s2323", "avr2",
                          20000000, 128, 128, false, 0, 2048 ) );
    m_cpuFeatures.append( KLCPUFeatures( "AT90S2333", "at90s2333", "avr2",
                          20000000, 128, 128, false, 0, 2048 ) );
    m_cpuFeatures.append( KLCPUFeatures( "AT90S2343", "at90s2343", "avr2",
                          20000000, 128, 128, false, 0, 2048 ) );
    m_cpuFeatures.append( KLCPUFeatures( "ATTiny22", "attiny22", "avr2",
                          12000000, 32, 64, false, 0, 1024 ) );
    m_cpuFeatures.append( KLCPUFeatures( "ATTiny24", "attiny24", "avr2",
         2048 , 128 , 128 , 12 , 20000000 , NO_RAM,  1.8, 5.5,
         1, 1, 4, false, false, false, true, true, 8, true, true,
         true, true, false, 17, 12, true, "MLF 20") );
    m_cpuFeatures.append( KLCPUFeatures( "ATTiny25", "attiny25", "avr2",
         2048 , 128 , 128 , 6 , 20000000 , NO_RAM,  1.8, 5.5, false,
         2, 4, false, false, false, true, true, 4, true, true, true,
         true, false, 15, 7, true, "MLF 20 MLF 20") );
    m_cpuFeatures.append( KLCPUFeatures( "ATTiny26", "attiny26", "avr2",
         2048 , 128 , 128 , 16 , 16000000 , NO_RAM,  2.7, 5.5, false, 2, 2,
         false, false, false, true, true, 11, true, true, true, true, false,
         11, 1, false, "SOIC 20") );
    m_cpuFeatures.append( KLCPUFeatures( "ATTiny261", "attiny261", "avr2",
                          12000000, 32, 64, false, 0, 1024 ) );
    m_cpuFeatures.append( KLCPUFeatures( "ATTiny44", "attiny44", "avr2",
         4096 , 256 , 256 , 12 , 20000000 , NO_RAM,  1.8, 5.5, 1, 1, 4,
         false, false, false, true, true, 8, true, true, true, true, false,
         17, 12, true, "MLF 20") );
    m_cpuFeatures.append( KLCPUFeatures( "ATTiny45", "attiny45", "avr2",
         4096 , 256 , 256 , 6 , 20000000 , NO_RAM,  1.8, 5.5, false, 2, 4,
         false, false, false, true, true, 4, true, true, true, true, false,
         15, 7, true, "MLF 20 MLF 20") );
    m_cpuFeatures.append( KLCPUFeatures( "ATTiny461", "attiny461", "avr2",
                          12000000, 32, 64, false, 0, 1024 ) );
    m_cpuFeatures.append( KLCPUFeatures( "ATTiny84", "attiny84", "avr2",
         8192 , 512 , 512 , 12 , 20000000 , NO_RAM,  1.8, 5.5, 1, 1, 4, false,
         false, false, true, true, 8, true, true, true, true, false, 17, 12,
         true, "MLF 20") );
    m_cpuFeatures.append( KLCPUFeatures( "ATTiny85", "attiny85", "avr2",
         8192 , 512 , 512 , 6 , 20000000 , NO_RAM,  1.8, 5.5, false, 2, 4,
         false, false, false, true, true, 4, true, true, true, true, false,
         15, 7, true, "MLF 20 MLF 20") );
    m_cpuFeatures.append( KLCPUFeatures( "ATTiny861", "attiny861", "avr2",
                          20000000, 512, 512, false, 0, 1024 ) );
    m_cpuFeatures.append( KLCPUFeatures( "AT90S4414", "at90s4414", "avr2",
                          8000000, 128, 256, false, 0, 4*1024 ) );
    m_cpuFeatures.append( KLCPUFeatures( "AT90S4433", "at90s4433", "avr2",
                          8000000, 128, 256, false, 0, 4*1024 ) );
    m_cpuFeatures.append( KLCPUFeatures( "AT90S4434", "at90s4434", "avr2",
                          8000000, 128, 256, false, 0, 4*1024 ) );
    m_cpuFeatures.append( KLCPUFeatures( "AT90S8515", "at90s8515", "avr2",
         8192 , 512 , 512 , 35 , 16000000 , NO_RAM,  2.7, 5.5, 1, 1, 3,
         false, 1, 1, false, true, false, false, true, true, true, true,
         16, 3, true, "PLCC 44 TQFP 44") );
    m_cpuFeatures.append( KLCPUFeatures( "AT90C8534", "at90c8534", "avr2",
                          12000000, 32, 64, false, 0, 1024 ) );
    m_cpuFeatures.append( KLCPUFeatures( "AT90S8535", "at90s8535", "avr2",
         8192 , 512 , 512 , 32 , 16000000 , NO_RAM,  2.7, 5.5, 1, 2, 4,
         false, 1, 1, true, true, 8, true, true, true, true, true, 20, 3,
         true, "PLCC 44 TQFP 44") );
    m_cpuFeatures.append( KLCPUFeatures( "AT86rf401", "at86rf401", "avr2",
                          12000000, 32, 64, false, 0, 1024 ) );
    m_cpuFeatures.append( KLCPUFeatures( "ATTiny13", "attiny13", "avr2",
         1024 , 64 , 96 , 6 , 20000000 , NO_RAM,  1.8, 5.5, false, 1, 2,
         false, false, false, false, true, 4, true, true, true, true,
         false, 9, 6, true, "MLF 20") );
    m_cpuFeatures.append( KLCPUFeatures( "ATTiny2313", "attiny2313", "avr2",
         2048 , 128 , 128 , 18 , 20000000 , NO_RAM,  1.8, 5.5, 1, 1, 4,
         false, false, 1, true, true, false, true, true, true, true,
         false, 8, 2, true, "MLF 20") );
    m_cpuFeatures.append( KLCPUFeatures( "ATMega103", "atmega103", "avr3",
                          16000000, 4096, 4096, true, 1024*64, 128*1024 ) );
    m_cpuFeatures.append( KLCPUFeatures( "ATMega603", "atmega603", "avr3",
                          16000000, 8192, 4096, true, 1024*64, 64*1024 ) );
    m_cpuFeatures.append( KLCPUFeatures( "AT43USB320", "at43usb320", "avr3",
                          12000000, 32, 64, false, 0, 1024 ) );
    m_cpuFeatures.append( KLCPUFeatures( "AT43USB355", "at43usb355", "avr3",
                          12000000, 32, 64, false, 0, 1024 ) );
    m_cpuFeatures.append( KLCPUFeatures( "AT76C711", "at76c711", "avr3",
                          12000000, 32, 64, false, 0, 1024 ) );
    m_cpuFeatures.append( KLCPUFeatures( "ATMega48", "atmega48", "avr4",
         4096 , 256 , 512 , 23 , 20000000 , NO_RAM,  1.8, 5.5, 1, 2, 6,
         true, 1, 1, true, true, 8, true, true, true, true, true, 26, 26,
         true, "TQFP 32") );
    m_cpuFeatures.append( KLCPUFeatures( "ATMega8", "atmega8", "avr4",
         8192 , 512 , 1024 , 23 , 16000000 , NO_RAM,  2.7, 5.5, 1, 2, 3,
         true, 1, 1, true, true, 8, true, true, true, true, true, 18, 2,
         true, "TQFP 32") );
    m_cpuFeatures.append( KLCPUFeatures( "ATMega8515", "atmega8515", "avr4",
         8192 , 512 , 512 , 35 , 16000000 , NO_RAM,  2.7, 5.5, 1, 1, 3,
         false, 1, 1, false, true, false, false, true, true, true, true,
         16, 3, true, "PLCC 44 TQFP 44") );
    m_cpuFeatures.append( KLCPUFeatures( "ATMega8535", "atmega8535", "avr4",
         8192 , 512 , 512 , 32 , 16000000 , NO_RAM,  2.7, 5.5, 1, 2, 4,
         false, 1, 1, true, true, 8, true, true, true, true, true, 20, 3,
         true, "PLCC 44 TQFP 44") );
    m_cpuFeatures.append( KLCPUFeatures( "ATMega88", "atmega88", "avr4",
         8192 , 512 , 1024 , 23 , 20000000 , NO_RAM,  1.8, 5.5, 1, 2, 6,
         true, 1, 1, true, true, 8, true, true, true, true, true, 26, 26,
         true, "TQFP 32") );
    m_cpuFeatures.append( KLCPUFeatures( "AT90PWM2", "at90pwm2", "avr4",
         8192 , 512 , 512 , 19 , 16000000 , NO_RAM,  2.7, 5.5, 1, 1, 7,
         true, 1, true, false, true, 8, true, true, true, true, true, false,
         4, true, "rrow SOIC 24 rrow SOIC 24") );
    m_cpuFeatures.append( KLCPUFeatures( "AT90PWM3", "at90pwm3", "avr4",
         8192 , 512 , 512 , 27 , 16000000 , NO_RAM,  2.7, 5.5, 1, 1, 10,
         true, 1, true, false, true, 11, true, true, true, true, true, false,
         4, true, "SOIC (440mil) 32") );
    m_cpuFeatures.append( KLCPUFeatures( "AT90CAN32", "at90can32", "avr5",
                          16000000, 2048, 1024, false, 0, 32*1024 ) );
    m_cpuFeatures.append( KLCPUFeatures( "AT90CAN64", "at90can64", "avr5",
                          16000000, 4096, 2048, false, 0, 64*1024 ) );
    m_cpuFeatures.append( KLCPUFeatures( "AT90CAN128", "at90can128", "avr5",
                          16000000, 4096, 4096, false, 0, 128*1024 ) );
    m_cpuFeatures.append( KLCPUFeatures( "AT90CAN646", "at90usb646", "avr5",
                          16000000, 4*1024, 2*1024, false, 0, 64*1024 ) );
    m_cpuFeatures.append( KLCPUFeatures( "AT90USB647", "at90usb647", "avr5",
                          16000000, 4*1024, 2*1024, false, 0, 64*1024 ) );
    m_cpuFeatures.append( KLCPUFeatures( "AT90USB1286", "at90usb1286", "avr5",
                          16000000, 1024*8, 1024*4, false, 0, 128*1024 ) );
    m_cpuFeatures.append( KLCPUFeatures( "AT90USB1287", "at90usb1287", "avr5",
                          16000000, 1024*8, 1024*4, false, 0, 128*1024 ) );
    m_cpuFeatures.append( KLCPUFeatures( "ATMega128", "atmega128", "avr5",
         131072 , 4096 , 4096 , 53 , 16000000 , NO_RAM,  2.7, 5.5, 2, 2, 8,
         true, 1, 2, true, true, 8, true, true, true, true, true, 34, 8, true,
         "TQFP 64") );
    m_cpuFeatures.append( KLCPUFeatures( "ATMega1280", "atmega1280", "avr5",
         131072 , 4096 , 8192 , 86 , 16000000 , NO_RAM,  1.8, 5.5, 4, 2, 16,
         true, 1, 4, true, true, 16, true, true, true, true, true, 57, 32, true,
         "TQFP 100 TQFP 100") );
    m_cpuFeatures.append( KLCPUFeatures( "ATMega1281", "atmega1281", "avr5",
         131072 , 4096 , 8192 , 54 , 16000000 , NO_RAM,  1.8, 5.5, 4, 2, 9,
         true, 1, 2, true, true, 8, true, true, true, true, true, 48, 17, true,
         "TQFP 64 MLF 64 TQFP 64") );
    m_cpuFeatures.append( KLCPUFeatures( "ATMega16", "atmega16", "avr5",
         16384 , 512 , 1024 , 32 , 16000000 , NO_RAM,  2.7, 5.5, 1, 2, 4,
         true, 1, 1, true, true, 8, true, true, true, true, true, 20, 3,
         true, "TQFP 44") );
    m_cpuFeatures.append( KLCPUFeatures( "ATMega161", "atmega161", "avr5",
                          12000000, 32, 64, false, 0, 1024 ) );
    m_cpuFeatures.append( KLCPUFeatures( "ATMega162", "atmega162", "avr5",
         16384 , 512 , 1024 , 35 , 16000000 , NO_RAM,  1.8, 5.5, 2, 2, 6,
         true, 1, 2, false, true, false, true, true, true, true, true,
         28, 3, true, "TQFP 44") );
    m_cpuFeatures.append( KLCPUFeatures( "ATMega163", "atmega163", "avr5",
                          12000000, 32, 64, false, 0, 1024 ) );
    m_cpuFeatures.append( KLCPUFeatures( "ATMega164p", "atmega164p", "avr5",
         16384 , 512 , 1024 , 32 , 20000000 , NO_RAM,  1.8, 5.5, 1, 2, 6, true,
         1, 2, true, true, 8, true, true, true, true, true, 31, 32,
         true, "TQFP 44") );
    m_cpuFeatures.append( KLCPUFeatures( "ATMega165", "atmega165", "avr5",
         16384 , 512 , 1024 , 54 , 16000000 , NO_RAM,  1.8, 5.5, 1, 2, 4,
         true, 1, 1, true, true, 8, true, true, true, true, true, 23, 17,
         true, "TQFP 64") );
    m_cpuFeatures.append( KLCPUFeatures( "ATMega168", "atmega168", "avr5",
         16384 , 512 , 1024 , 23 , 20000000 , NO_RAM,  1.8, 5.5, 1, 2, 6,
         true, 1, 1, true, true, 8, true, true, true, true, true, 26, 26,
         true, "TQFP 32") );
    m_cpuFeatures.append( KLCPUFeatures( "ATMega169", "atmega169", "avr5",
         16384 , 512 , 1024 , 54 , 16000000 , NO_RAM,  1.8, 5.5, 1, 2, 4,
         true, 1, 1, true, true, 8, true, true, true, true, true, 23, 17,
         true, "TQFP 64") );
    m_cpuFeatures.append( KLCPUFeatures( "ATMega32", "atmega32", "avr5",
         32768 , 1024 , 2048 , 32 , 16000000 , NO_RAM,  2.7, 5.5, 1, 2, 4,
         true, 1, 1, true, true, 8, true, true, true, true, true, 19, 3,
         true, "TQFP 44") );
    m_cpuFeatures.append( KLCPUFeatures( "ATMega323", "atmega323", "avr5",
                          12000000, 32, 64, false, 0, 1024 ) );
    m_cpuFeatures.append( KLCPUFeatures( "ATMega324p", "atmega324p", "avr5",
         32768 , 1024 , 2048 , 32 , 20000000 , NO_RAM,  1.8, 5.5, 1, 2, 6,
         true, 1, 2, true, true, 8, true, true, true, true, true, 31, 32,
         true, "TQFP 44") );
    m_cpuFeatures.append( KLCPUFeatures( "ATMega325", "atmega325", "avr5",
         32768 , 1024 , 2048 , 54 , 16000000 , NO_RAM,  1.8, 5.5, 1, 2, 4,
         true, 1, 1, true, true, 8, true, true, true, true, true, 23, 17,
         true, "TQFP 64") );
    m_cpuFeatures.append( KLCPUFeatures( "ATMega3250", "atmega3250", "avr5",
         32768 , 1024 , 2048 , 69 , 16000000 , NO_RAM,  1.8, 5.5, 1, 2, 4,
         true, 1, 1, true, true, 8, true, true, true, true, true, 32, 17,
         true, "MLF 64 TQFP 64") );
    m_cpuFeatures.append( KLCPUFeatures( "ATMega329", "atmega329", "avr5",
         32768 , 1024 , 2048 , 54 , 16000000 , NO_RAM,  1.8, 5.5, 1, 2, 4,
         true, 1, 1, true, true, 8, true, true, true, true, true, 25, 17,
         true, "TQFP 64") );
    m_cpuFeatures.append( KLCPUFeatures( "ATMega3290", "atmega3290", "avr5",
         32768 , 1024 , 2048 , 69 , 16000000 , NO_RAM,  1.8, 5.5, 1, 2, 4,
         true, 1, 1, true, true, 8, true, true, true, true, true, 25, 32,
         true, "MLF 64 TQFP 64") );
    m_cpuFeatures.append( KLCPUFeatures( "ATMega406", "atmega406", "avr5",
         40960 , 512 , 2048 , 18 , 1000000 , NO_RAM,  1.8, 5.5, 1, 1, 1,
         true, false, false, true, true, false, true, true, true, true,
         true, 23, 4, true, "LQFP 48 LQFP 48") );
    m_cpuFeatures.append( KLCPUFeatures( "ATMega64", "atmega64", "avr5",
         65536 , 2048 , 4096 , 54 , 16000000 , NO_RAM,  2.7, 5.5, 2, 2, 8,
         true, 1, 2, true, true, 8, true, true, true, true, true, 34, 8,
         true, "TQFP 64") );
    m_cpuFeatures.append( KLCPUFeatures( "ATMega640", "atmega640", "avr5",
         65536 , 4000 , 8192 , 86 , 16000000 , NO_RAM,  1.8, 5.5, 4, 2, 16,
         true, 1, 4, true, true, 16, true, true, true, true, true, 57, 32,
         true, "TQFP 100 TQFP 100") );
    m_cpuFeatures.append( KLCPUFeatures( "ATMega644", "atmega644", "avr5",
         65536 , 2048 , 4096 , 32 , 20000000 , NO_RAM,  1.8, 5.5, 1, 2, 6,
         true, 1, 1, true, true, 8, true, true, true, true, true, 31, 32,
         true, "TQFP 44") );
    m_cpuFeatures.append( KLCPUFeatures( "ATMega644p", "atmega644p", "avr5",
         65536 , 2048 , 4096 , 32 , 20000000 , NO_RAM,  1.8, 5.5, 1, 2, 6,
         true, 1, 1, true, true, 8, true, true, true, true, true, 31, 32,
         true, "TQFP 44") );
    m_cpuFeatures.append( KLCPUFeatures( "ATMega645", "atmega645", "avr5",
         65536 , 2048 , 4096 , 54 , 16000000 , NO_RAM,  1.8, 5.5, 1, 2, 4,
         true, 1, 1, true, true, 8, true, true, true, true, true, 23, 17,
         true, "TQFP 64") );
    m_cpuFeatures.append( KLCPUFeatures( "ATMega6450", "atmega6450", "avr5",
         65536 , 2048 , 4096 , 69 , 16000000 , NO_RAM,  1.8, 5.5, 1, 2, 4,
         true, 1, 1, true, true, 8, true, true, true, true, true, 32, 17,
         true, "MLF 64 TQFP 64") );
    m_cpuFeatures.append( KLCPUFeatures( "ATMega649", "atmega649", "avr5",
         65536 , 2048 , 4096 , 54 , 16000000 , NO_RAM,  1.8, 5.5, 1, 2, 4,
         true, 1, 1, true, true, 8, true, true, true, true, true, 25, 17,
         true, "TQFP 64") );
    m_cpuFeatures.append( KLCPUFeatures( "ATMega6490", "atmega6490", "avr5",
         65536 , 2048 , 4096 , 69 , 16000000 , NO_RAM,  1.8, 5.5, 1, 2, 4,
         true, 1, 1, true, true, 8, true, true, true, true, true, 25, 32,
         true, "MLF 64 TQFP 64") );
    m_cpuFeatures.append( KLCPUFeatures( "AT94k", "at94k", "avr5",
                          12000000, 32, 64, false, 0, 1024 ) );
    
    qHeapSort( m_cpuFeatures );
    
    QValueList<KLCPUFeatures>::iterator it;
    for (it=m_cpuFeatures.begin(); it!=m_cpuFeatures.end(); ++it)
        m_cpus.append((*it).name());
    
    // Here, we define the meaning of the fuse bits:
    QValueList< bool > lowCanBeChanged;
    QValueList< bool > highCanBeChanged;
    QValueList< bool > extCanBeChanged;
    QStringList lowNames;
    QStringList highNames;
    QStringList extNames;
    
    lowCanBeChanged.clear();
    highCanBeChanged.clear();
    extCanBeChanged.clear();
    lowNames.clear();
    highNames.clear();
    extNames.clear();
    lowCanBeChanged << true << true << true << true << true << true << true << true;
    highCanBeChanged << true << true << true << true << true << true << true << true;
    extCanBeChanged << true << true << true << true << false << false << false << false;
    lowNames << "CKSEL0" << "CKSEL1" << "CKSEL2" << "CKSEL3" << "SUT0" << "SUT1" << "CKOUT" << "CKDIV8";
    highNames << "BOOTRST" << "BOOTSz0" << "BOOTSZ1" << "EESAVE" << "WDTON" << "SPIEN" << "JTAGEN" << "OCDEN";
    extNames << "RSTDISBL" << "BODLEVEL0" << "BODLEVEL1" << "BODLEVEL2" << "1" << "1" << "1" << "1";
    m_cpuFuses.append( KLCPUFuses( "ATmega169", lowCanBeChanged, highCanBeChanged,
                       extCanBeChanged, lowNames, highNames, extNames ) );

    lowCanBeChanged.clear();
    highCanBeChanged.clear();
    extCanBeChanged.clear();
    lowNames.clear();
    highNames.clear();
    extNames.clear();
    lowCanBeChanged << true << false << false << false << false << true << false << false;
    highCanBeChanged << false << false << false << false << false << false << false << false;
    extCanBeChanged << false << false << false << false << false << false << false << false;
    lowNames << "FSTRT" << "1" << "1" << "1" << "1" << "SIEN" << "1" << "1";
    highNames << "1" << "1" << "1" << "1" << "1" << "1" << "1" << "1";
    extNames << "-" << "-" << "-" << "-" << "-" << "-" << "-" << "-";
    m_cpuFuses.append( KLCPUFuses( "AT90S8515", lowCanBeChanged, highCanBeChanged,
                       extCanBeChanged, lowNames, highNames, extNames ) );

    lowCanBeChanged.clear();
    highCanBeChanged.clear();
    extCanBeChanged.clear();
    lowNames.clear();
    highNames.clear();
    extNames.clear();
    lowCanBeChanged << true << true << true << true << true << true << true << false;
    highCanBeChanged << false << false << false << false << false << false << false << false;
    extCanBeChanged << false << false << false << false << false << false << false << false;
    lowNames << "CKSEL0" << "CKSEL1" << "CKSEL2" << "BODEN" << "BODLVL" << "SPIEN" << "BOOTRST" << "1";
    highNames << "1" << "1" << "1" << "1" << "1" << "1" << "1" << "1";
    extNames << "-" << "-" << "-" << "-" << "-" << "-" << "-" << "-";
    m_cpuFuses.append( KLCPUFuses( "ATmega161", lowCanBeChanged, highCanBeChanged,
                       extCanBeChanged, lowNames, highNames, extNames ) );

    lowCanBeChanged.clear();
    highCanBeChanged.clear();
    extCanBeChanged.clear();
    lowNames.clear();
    highNames.clear();
    extNames.clear();
    lowCanBeChanged << true << true << true << true << false << true << true << true;
    highCanBeChanged << true << true << true << false << false << false << false << false;
    extCanBeChanged << false << false << false << false << false << false << false << false;
    lowNames << "CKSEL0" << "CKSEL1" << "CKSEL2" << "CKSEL3" << "1" << "SPIEN" << "BODEN" << "BODLEV";
    highNames << "BOOTRST" << "TOOTSZ0" << "BOOTSZ1" << "1" << "1" << "1" << "1" << "1";
    extNames << "-" << "-" << "-" << "-" << "-" << "-" << "-" << "-";
    m_cpuFuses.append( KLCPUFuses( "ATmega163", lowCanBeChanged, highCanBeChanged,
                       extCanBeChanged, lowNames, highNames, extNames ) );

    lowCanBeChanged.clear();
    highCanBeChanged.clear();
    extCanBeChanged.clear();
    lowNames.clear();
    highNames.clear();
    extNames.clear();
    lowCanBeChanged << true << false << false << false << false << true << false << false;
    highCanBeChanged << false << false << false << false << false << false << false << false;
    extCanBeChanged << false << false << false << false << false << false << false << false;
    lowNames << "FSTRT" << "1" << "1" << "1" << "1" << "SPIEN" << "1" << "1";
    highNames << "1" << "1" << "1" << "1" << "1" << "1" << "1" << "1";
    extNames << "-" << "-" << "-" << "-" << "-" << "-" << "-" << "-";
    m_cpuFuses.append( KLCPUFuses( "AT90S8535", lowCanBeChanged, highCanBeChanged,
                       extCanBeChanged, lowNames, highNames, extNames ) );

    lowCanBeChanged.clear();
    highCanBeChanged.clear();
    extCanBeChanged.clear();
    lowNames.clear();
    highNames.clear();
    extNames.clear();
    lowCanBeChanged << true << true << true << true << false << false << true << true;
    highCanBeChanged << true << true << true << true << false << true << true << true;
    extCanBeChanged << false << false << false << false << false << false << false << false;
    lowNames << "CKSEL0" << "CKSEL1" << "CKSEL2" << "CKSEL3" << "1" << "1" << "BODEN" << "BODLEV";
    highNames << "BOOTRST" << "TOOTSZ0" << "BOOTSZ1" << "EESAVE" << "1" << "SPIEN" << "JTAGEN" << "OCDEN";
    extNames << "-" << "-" << "-" << "-" << "-" << "-" << "-" << "-";
    m_cpuFuses.append( KLCPUFuses( "ATmega323", lowCanBeChanged, highCanBeChanged,
                       extCanBeChanged, lowNames, highNames, extNames ) );

    lowCanBeChanged.clear();
    highCanBeChanged.clear();
    extCanBeChanged.clear();
    lowNames.clear();
    highNames.clear();
    extNames.clear();
    lowCanBeChanged << true << true << true << true << true << true << true << true;
    highCanBeChanged << false << false << false << false << true << true << true << true;
    extCanBeChanged << true << true << false << false << false << false << false << false;
    lowNames << "CKSEL0" << "CKSEL1" << "CKSEL2" << "CKSEL3" << "SUT0" << "SUT1" << "BODEN" << "BODLEVEL";
    highNames << " " << " " << " " << " " << "CKOPT" << "SPIEN" << "JTAGEN" << "OCDEN";
    extNames << "WDTON" << "M103C" << "1" << "1" << "1" << "1" << "1" << "1";
    m_cpuFuses.append( KLCPUFuses( "ATmega128", lowCanBeChanged, highCanBeChanged,
                       extCanBeChanged, lowNames, highNames, extNames ) );

    lowCanBeChanged.clear();
    highCanBeChanged.clear();
    extCanBeChanged.clear();
    lowNames.clear();
    highNames.clear();
    extNames.clear();
    lowCanBeChanged << true << false << false << false << false << true << false << false;
    highCanBeChanged << false << false << false << false << false << false << false << false;
    extCanBeChanged << false << false << false << false << false << false << false << false;
    lowNames << "RCEN" << "1" << "1" << "1" << "1" << "SPIEN" << "1" << "1";
    highNames << "1" << "1" << "1" << "1" << "1" << "1" << "1" << "1";
    extNames << "-" << "-" << "-" << "-" << "-" << "-" << "-" << "-";
    m_cpuFuses.append( KLCPUFuses( "AT90S1200", lowCanBeChanged, highCanBeChanged,
                       extCanBeChanged, lowNames, highNames, extNames ) );

    lowCanBeChanged.clear();
    highCanBeChanged.clear();
    extCanBeChanged.clear();
    lowNames.clear();
    highNames.clear();
    extNames.clear();
    lowCanBeChanged << true << true << true << true << true << true << true << true;
    highCanBeChanged << true << true << true << true << true << true << true << true;
    extCanBeChanged << false << false << false << false << false << false << false << false;
    lowNames << "CKSEL0" << "CKSEL1" << "CKSEL2" << "CKSEL3" << "SUT0" << "SUT1" << "BODEN" << "BODLEVEL";
    highNames << "BOOTRST" << "BOOTSz0" << "BOOTSZ1" << "EESAVE" << "CKOPT" << "SPIEN" << "JTAGEN" << "OCDEN";
    extNames << "-" << "-" << "-" << "-" << "-" << "-" << "-" << "-";
    m_cpuFuses.append( KLCPUFuses( "ATmega16", lowCanBeChanged, highCanBeChanged,
                       extCanBeChanged, lowNames, highNames, extNames ) );

    lowCanBeChanged.clear();
    highCanBeChanged.clear();
    extCanBeChanged.clear();
    lowNames.clear();
    highNames.clear();
    extNames.clear();
    lowCanBeChanged << true << true << true << true << true << true << true << true;
    highCanBeChanged << true << true << true << true << true << true << true << true;
    extCanBeChanged << false << false << false << false << false << false << false << false;
    lowNames << "CKSEL0" << "CKSEL1" << "CKSEL2" << "CKSEL3" << "SUT0" << "SUT1" << "BODEN" << "BODLEVEL";
    highNames << "BOOTRST" << "BOOTSz0" << "BOOTSZ1" << "EESAVE" << "CKOPT" << "SPIEN" << "JTAGEN" << "OCDEN";
    extNames << "-" << "-" << "-" << "-" << "-" << "-" << "-" << "-";
    m_cpuFuses.append( KLCPUFuses( "ATmega32", lowCanBeChanged, highCanBeChanged,
                       extCanBeChanged, lowNames, highNames, extNames ) );

    lowCanBeChanged.clear();
    highCanBeChanged.clear();
    extCanBeChanged.clear();
    lowNames.clear();
    highNames.clear();
    extNames.clear();
    lowCanBeChanged << true << false << false << false << false << true << false << false;
    highCanBeChanged << false << false << false << false << false << false << false << false;
    extCanBeChanged << false << false << false << false << false << false << false << false;
    lowNames << "FSTRT" << "1" << "1" << "1" << "1" << "SPIEN" << "1" << "1";
    highNames << "1" << "1" << "1" << "1" << "1" << "1" << "1" << "1";
    extNames << "-" << "-" << "-" << "-" << "-" << "-" << "-" << "-";
    m_cpuFuses.append( KLCPUFuses( "AT90S2313", lowCanBeChanged, highCanBeChanged,
                       extCanBeChanged, lowNames, highNames, extNames ) );

    lowCanBeChanged.clear();
    highCanBeChanged.clear();
    extCanBeChanged.clear();
    lowNames.clear();
    highNames.clear();
    extNames.clear();
    lowCanBeChanged << true << true << true << true << true << true << false << false;
    highCanBeChanged << false << false << false << false << false << false << false << false;
    extCanBeChanged << false << false << false << false << false << false << false << false;
    lowNames << "CKSEL0" << "CKSEL1" << "CKSEL2" << "BODEN" << "BODLVL" << "SPIEN" << "1" << "1";
    highNames << "1" << "1" << "1" << "1" << "1" << "1" << "1" << "1";
    extNames << "-" << "-" << "-" << "-" << "-" << "-" << "-" << "-";
    m_cpuFuses.append( KLCPUFuses( "AT90S4433", lowCanBeChanged, highCanBeChanged,
                       extCanBeChanged, lowNames, highNames, extNames ) );

    lowCanBeChanged.clear();
    highCanBeChanged.clear();
    extCanBeChanged.clear();
    lowNames.clear();
    highNames.clear();
    extNames.clear();
    lowCanBeChanged << true << true << true << true << true << true << false << false;
    highCanBeChanged << false << false << false << false << false << false << false << false;
    extCanBeChanged << false << false << false << false << false << false << false << false;
    lowNames << "CKSEL0" << "CKSEL1" << "CKSEL2" << "BODEN" << "BODLVL" << "SPIEN" << "1" << "1";
    highNames << "1" << "1" << "1" << "1" << "1" << "1" << "1" << "1";
    extNames << "-" << "-" << "-" << "-" << "-" << "-" << "-" << "-";
    m_cpuFuses.append( KLCPUFuses( "AT90S2333", lowCanBeChanged, highCanBeChanged,
                       extCanBeChanged, lowNames, highNames, extNames ) );

    lowCanBeChanged.clear();
    highCanBeChanged.clear();
    extCanBeChanged.clear();
    lowNames.clear();
    highNames.clear();
    extNames.clear();
    lowCanBeChanged << true << false << false << false << false << true << false << false;
    highCanBeChanged << false << false << false << false << false << false << false << false;
    extCanBeChanged << false << false << false << false << false << false << false << false;
    lowNames << "FSTRT" << "1" << "1" << "1" << "1" << "SPIEN" << "1" << "1";
    highNames << "1" << "1" << "1" << "1" << "1" << "1" << "1" << "1";
    extNames << "-" << "-" << "-" << "-" << "-" << "-" << "-" << "-";
    m_cpuFuses.append( KLCPUFuses( "AT90S4434", lowCanBeChanged, highCanBeChanged,
                       extCanBeChanged, lowNames, highNames, extNames ) );

    lowCanBeChanged.clear();
    highCanBeChanged.clear();
    extCanBeChanged.clear();
    lowNames.clear();
    highNames.clear();
    extNames.clear();
    lowCanBeChanged << true << false << false << false << false << true << false << false;
    highCanBeChanged << false << false << false << false << false << false << false << false;
    extCanBeChanged << false << false << false << false << false << false << false << false;
    lowNames << "FSTRT" << "1" << "1" << "1" << "1" << "SPIEN" << "1" << "1";
    highNames << "1" << "1" << "1" << "1" << "1" << "1" << "1" << "1";
    extNames << "-" << "-" << "-" << "-" << "-" << "-" << "-" << "-";
    m_cpuFuses.append( KLCPUFuses( "AT90S4414", lowCanBeChanged, highCanBeChanged,
                       extCanBeChanged, lowNames, highNames, extNames ) );

    lowCanBeChanged.clear();
    highCanBeChanged.clear();
    extCanBeChanged.clear();
    lowNames.clear();
    highNames.clear();
    extNames.clear();
    lowCanBeChanged << true << true << false << false << true << true << true << true;
    highCanBeChanged << false << false << false << false << false << false << false << false;
    extCanBeChanged << false << false << false << false << false << false << false << false;
    lowNames << "CKSEL0" << "CKSEL1" << "1" << "1" << "RTDSBL" << "SPIEN" << "BODEN" << "BODLVL";
    highNames << "1" << "1" << "1" << "1" << "1" << "1" << "1" << "1";
    extNames << "-" << "-" << "-" << "-" << "-" << "-" << "-" << "-";
    m_cpuFuses.append( KLCPUFuses( "ATtiny15", lowCanBeChanged, highCanBeChanged,
                       extCanBeChanged, lowNames, highNames, extNames ) );

    lowCanBeChanged.clear();
    highCanBeChanged.clear();
    extCanBeChanged.clear();
    lowNames.clear();
    highNames.clear();
    extNames.clear();
    lowCanBeChanged << true << true << true << true << true << true << true << true;
    highCanBeChanged << false << false << false << false << false << false << false << false;
    extCanBeChanged << false << false << false << false << false << false << false << false;
    lowNames << "CKSEL0" << "CKSEL1" << "CKSEL2" << "CKSEL3" << "RSTDSBL" << "SPIEN" << "BODEN" << "BODLVL";
    highNames << "1" << "1" << "1" << "1" << "1" << "1" << "1" << "1";
    extNames << "-" << "-" << "-" << "-" << "-" << "-" << "-" << "-";
    m_cpuFuses.append( KLCPUFuses( "ATtiny12", lowCanBeChanged, highCanBeChanged,
                       extCanBeChanged, lowNames, highNames, extNames ) );

    lowCanBeChanged.clear();
    highCanBeChanged.clear();
    extCanBeChanged.clear();
    lowNames.clear();
    highNames.clear();
    extNames.clear();
    lowCanBeChanged << true << true << true << true << true << true << true << true;
    highCanBeChanged << false << false << false << false << false << false << false << false;
    extCanBeChanged << false << false << false << false << false << false << false << false;
    lowNames << "CKSEL0" << "CKSEL1" << "CKSEL2" << "RSTDSBL" << "FSTRT" << "0" << "0" << "0";
    highNames << "1" << "1" << "1" << "1" << "1" << "1" << "1" << "1";
    extNames << "-" << "-" << "-" << "-" << "-" << "-" << "-" << "-";
    m_cpuFuses.append( KLCPUFuses( "ATtiny11", lowCanBeChanged, highCanBeChanged,
                       extCanBeChanged, lowNames, highNames, extNames ) );

    lowCanBeChanged.clear();
    highCanBeChanged.clear();
    extCanBeChanged.clear();
    lowNames.clear();
    highNames.clear();
    extNames.clear();
    lowCanBeChanged << true << true << true << true << true << true << true << true;
    highCanBeChanged << true << true << true << true << true << true << true << true;
    extCanBeChanged << false << true << true << true << true << false << false << false;
    lowNames << "CKSEL0" << "CKSEL1" << "CKSEL2" << "CKSEL3" << "SUT0" << "SUT1" << "CKOUT" << "CKDIV8";
    highNames << "BOOTRST" << "BOOTSz0" << "BOOTSZ1" << "EESAVE" << "WDTON" << "SPIEN" << "JTAGEN" << "OCDEN";
    extNames << "1" << "BODLEVEL0" << "BODLEVEL1" << "BODLEVEL2" << "M161C" << "1" << "1" << "1";
    m_cpuFuses.append( KLCPUFuses( "ATmega162", lowCanBeChanged, highCanBeChanged,
                       extCanBeChanged, lowNames, highNames, extNames ) );

    lowCanBeChanged.clear();
    highCanBeChanged.clear();
    extCanBeChanged.clear();
    lowNames.clear();
    highNames.clear();
    extNames.clear();
    lowCanBeChanged << false << false << false << false << false << false << false << false;
    highCanBeChanged << false << false << false << false << false << false << false << false;
    extCanBeChanged << false << false << false << false << false << false << false << false;
    lowNames << "1" << "1" << "1" << "1" << "1" << "1" << "1" << "1";
    highNames << "1" << "1" << "1" << "1" << "1" << "1" << "1" << "1";
    extNames << "-" << "-" << "-" << "-" << "-" << "-" << "-" << "-";
    m_cpuFuses.append( KLCPUFuses( "AT89552", lowCanBeChanged, highCanBeChanged,
                       extCanBeChanged, lowNames, highNames, extNames ) );

    lowCanBeChanged.clear();
    highCanBeChanged.clear();
    extCanBeChanged.clear();
    lowNames.clear();
    highNames.clear();
    extNames.clear();
    lowCanBeChanged << false << false << false << false << false << false << false << false;
    highCanBeChanged << false << false << false << false << false << false << false << false;
    extCanBeChanged << false << false << false << false << false << false << false << false;
    lowNames << "1" << "1" << "1" << "1" << "1" << "1" << "1" << "1";
    highNames << "1" << "1" << "1" << "1" << "1" << "1" << "1" << "1";
    extNames << "-" << "-" << "-" << "-" << "-" << "-" << "-" << "-";
    m_cpuFuses.append( KLCPUFuses( "AT89551", lowCanBeChanged, highCanBeChanged,
                       extCanBeChanged, lowNames, highNames, extNames ) );

    lowCanBeChanged.clear();
    highCanBeChanged.clear();
    extCanBeChanged.clear();
    lowNames.clear();
    highNames.clear();
    extNames.clear();
    lowCanBeChanged << true << false << false << false << false << true << false << false;
    highCanBeChanged << false << false << false << false << false << false << false << false;
    extCanBeChanged << false << false << false << false << false << false << false << false;
    lowNames << "RCEN" << "1" << "1" << "1" << "1" << "SPIEN" << "1" << "1";
    highNames << "1" << "1" << "1" << "1" << "1" << "1" << "1" << "1";
    extNames << "-" << "-" << "-" << "-" << "-" << "-" << "-" << "-";
    m_cpuFuses.append( KLCPUFuses( "AT90S2343", lowCanBeChanged, highCanBeChanged,
                       extCanBeChanged, lowNames, highNames, extNames ) );

    lowCanBeChanged.clear();
    highCanBeChanged.clear();
    extCanBeChanged.clear();
    lowNames.clear();
    highNames.clear();
    extNames.clear();
    lowCanBeChanged << true << true << false << true << false << true << false << false;
    highCanBeChanged << false << false << false << false << false << false << false << false;
    extCanBeChanged << false << false << false << false << false << false << false << false;
    lowNames << "SUT0" << "SUT1" << "1" << "EESAVE" << "1" << "SPIEN" << "1" << "1";
    highNames << "1" << "1" << "1" << "1" << "1" << "1" << "1" << "1";
    extNames << "-" << "-" << "-" << "-" << "-" << "-" << "-" << "-";
    m_cpuFuses.append( KLCPUFuses( "ATmega103", lowCanBeChanged, highCanBeChanged,
                       extCanBeChanged, lowNames, highNames, extNames ) );

    lowCanBeChanged.clear();
    highCanBeChanged.clear();
    extCanBeChanged.clear();
    lowNames.clear();
    highNames.clear();
    extNames.clear();
    lowCanBeChanged << true << true << true << true << true << true << true << true;
    highCanBeChanged << false << false << false << false << true << true << true << true;
    extCanBeChanged << true << true << false << false << false << false << false << false;
    lowNames << "CKSEL0" << "CKSEL1" << "CKSEL2" << "CKSEL3" << "SUT0" << "SUT1" << "BODEN" << "BODLEVEL";
    highNames << " " << " " << " " << " " << "CKOPT" << "SPIEN" << "JTAGEN" << "OCDEN";
    extNames << "WDTON" << "M103C" << "1" << "1" << "1" << "1" << "1" << "1";
    m_cpuFuses.append( KLCPUFuses( "ATmega64", lowCanBeChanged, highCanBeChanged,
                       extCanBeChanged, lowNames, highNames, extNames ) );

    lowCanBeChanged.clear();
    highCanBeChanged.clear();
    extCanBeChanged.clear();
    lowNames.clear();
    highNames.clear();
    extNames.clear();
    lowCanBeChanged << true << false << false << false << false << true << false << false;
    highCanBeChanged << false << false << false << false << false << false << false << false;
    extCanBeChanged << false << false << false << false << false << false << false << false;
    lowNames << "FSTRT" << "1" << "1" << "1" << "1" << "SPIEN" << "1" << "1";
    highNames << "1" << "1" << "1" << "1" << "1" << "1" << "1" << "1";
    extNames << "-" << "-" << "-" << "-" << "-" << "-" << "-" << "-";
    m_cpuFuses.append( KLCPUFuses( "AT90S2323", lowCanBeChanged, highCanBeChanged,
                       extCanBeChanged, lowNames, highNames, extNames ) );

    lowCanBeChanged.clear();
    highCanBeChanged.clear();
    extCanBeChanged.clear();
    lowNames.clear();
    highNames.clear();
    extNames.clear();
    lowCanBeChanged << false << false << false << false << false << false << false << false;
    highCanBeChanged << false << false << false << false << false << false << false << false;
    extCanBeChanged << false << false << false << false << false << false << false << false;
    lowNames << "1" << "1" << "1" << "1" << "1" << "1" << "1" << "1";
    highNames << "1" << "1" << "1" << "1" << "1" << "1" << "1" << "1";
    extNames << "-" << "-" << "-" << "-" << "-" << "-" << "-" << "-";
    m_cpuFuses.append( KLCPUFuses( "AT86RF401", lowCanBeChanged, highCanBeChanged,
                       extCanBeChanged, lowNames, highNames, extNames ) );

    lowCanBeChanged.clear();
    highCanBeChanged.clear();
    extCanBeChanged.clear();
    lowNames.clear();
    highNames.clear();
    extNames.clear();
    lowCanBeChanged << true << true << true << true << true << true << true << true;
    highCanBeChanged << true << true << true << true << true << true << true << true;
    extCanBeChanged << false << false << false << false << false << false << false << false;
    lowNames << "CKSEL0" << "CKSEL1" << "CKSEL2" << "CKSEL3" << "SUT0" << "SUT1" << "BODEN" << "BODLEVEL";
    highNames << "BOOTRST" << "BOOTSz0" << "BOOTSZ1" << "EESAVE" << "CKOPT" << "SPIEN" << "WDTON" << "RSTDISBL";
    extNames << "-" << "-" << "-" << "-" << "-" << "-" << "-" << "-";
    m_cpuFuses.append( KLCPUFuses( "ATmega8", lowCanBeChanged, highCanBeChanged,
                       extCanBeChanged, lowNames, highNames, extNames ) );

    lowCanBeChanged.clear();
    highCanBeChanged.clear();
    extCanBeChanged.clear();
    lowNames.clear();
    highNames.clear();
    extNames.clear();
    lowCanBeChanged << true << false << false << false << false << true << false << false;
    highCanBeChanged << false << false << false << false << false << false << false << false;
    extCanBeChanged << false << false << false << false << false << false << false << false;
    lowNames << "RCEN" << "1" << "1" << "1" << "1" << "SPIEN" << "1" << "1";
    highNames << "1" << "1" << "1" << "1" << "1" << "1" << "1" << "1";
    extNames << "-" << "-" << "-" << "-" << "-" << "-" << "-" << "-";
    m_cpuFuses.append( KLCPUFuses( "ATtiny22", lowCanBeChanged, highCanBeChanged,
                       extCanBeChanged, lowNames, highNames, extNames ) );

    lowCanBeChanged.clear();
    highCanBeChanged.clear();
    extCanBeChanged.clear();
    lowNames.clear();
    highNames.clear();
    extNames.clear();
    lowCanBeChanged << true << true << true << true << true << true << true << true;
    highCanBeChanged << true << true << true << true << true << true << true << true;
    extCanBeChanged << false << false << false << false << false << false << false << false;
    lowNames << "CKSEL0" << "CKSEL1" << "CKSEL2" << "CKSEL3" << "SUT0" << "SUT1" << "BODEN" << "BODLEVEL";
    highNames << "BOOTRST" << "BOOTSz0" << "BOOTSZ1" << "EESAVE" << "CKOPT" << "SPIEN" << "WDTON" << "S8535C";
    extNames << "-" << "-" << "-" << "-" << "-" << "-" << "-" << "-";
    m_cpuFuses.append( KLCPUFuses( "ATmega8535", lowCanBeChanged, highCanBeChanged,
                       extCanBeChanged, lowNames, highNames, extNames ) );

    lowCanBeChanged.clear();
    highCanBeChanged.clear();
    extCanBeChanged.clear();
    lowNames.clear();
    highNames.clear();
    extNames.clear();
    lowCanBeChanged << true << true << true << true << true << true << true << true;
    highCanBeChanged << true << true << true << true << true << false << false << false;
    extCanBeChanged << false << false << false << false << false << false << false << false;
    lowNames << "CKSEL0" << "CKSEL1" << "CKSEL2" << "CKSEL3" << "SUT0" << "SUT1" << "CKOPT" << "PLLCK";
    highNames << "BODEN" << "DOBLEVEL" << "EESAVE" << "SPIEN" << "RSTDISBL" << "1" << "1" << "1";
    extNames << "-" << "-" << "-" << "-" << "-" << "-" << "-" << "-";
    m_cpuFuses.append( KLCPUFuses( "ATtiny26", lowCanBeChanged, highCanBeChanged,
                       extCanBeChanged, lowNames, highNames, extNames ) );

    lowCanBeChanged.clear();
    highCanBeChanged.clear();
    extCanBeChanged.clear();
    lowNames.clear();
    highNames.clear();
    extNames.clear();
    lowCanBeChanged << true << true << true << true << true << true << true << true;
    highCanBeChanged << true << true << true << true << true << true << true << true;
    extCanBeChanged << false << false << false << false << false << false << false << false;
    lowNames << "CKSEL0" << "CKSEL1" << "CKSEL2" << "CKSEL3" << "SUT0" << "SUT1" << "BODEN" << "BODLEVEL";
    highNames << "BOOTRST" << "BOOTSz0" << "BOOTSZ1" << "EESAVE" << "CKOPT" << "SPIEN" << "WDTON" << "S8515C";
    extNames << "-" << "-" << "-" << "-" << "-" << "-" << "-" << "-";
    m_cpuFuses.append( KLCPUFuses( "ATmega8515", lowCanBeChanged, highCanBeChanged,
                       extCanBeChanged, lowNames, highNames, extNames ) );

    lowCanBeChanged.clear();
    highCanBeChanged.clear();
    extCanBeChanged.clear();
    lowNames.clear();
    highNames.clear();
    extNames.clear();
    lowCanBeChanged << true << true << true << true << true << false << false << false;
    highCanBeChanged << false << false << false << false << false << false << false << false;
    extCanBeChanged << false << false << false << false << false << false << false << false;
    lowNames << "CKSEL0" << "CKSEL1" << "CKSEL2" << "CKSEL3" << "INTCAP" << "1" << "1" << "1";
    highNames << "1" << "1" << "1" << "1" << "1" << "1" << "1" << "1";
    extNames << "-" << "-" << "-" << "-" << "-" << "-" << "-" << "-";
    m_cpuFuses.append( KLCPUFuses( "ATtiny28", lowCanBeChanged, highCanBeChanged,
                       extCanBeChanged, lowNames, highNames, extNames ) );

    
    m_compilerInterface = (KLCompilerInterface*) new KLAVRGCCCompiler( this, m_parent->procManager() );
}


KLProject::~KLProject()
{
}


void KLProject::addDocument( KLDocument * doc, bool forceCopyToProjectDir )
{
    if (!initialized())
        return;
    if ( m_documents.findRef( doc ) < 0 )
    {
        m_unsaved=true;
        m_documents.append( doc );
        doc->setProject( this );
        if (!m_projectBaseURL.isParentOf( doc->url() ))
        {
            if ( !forceCopyToProjectDir )
            KMessageBox::information( m_parent,
                                      i18n("The file is not in your " \
                                           "project base directory. It will be copied " \
                                           "to it."),
                                      i18n("File not relative to project base directory"),
                                      "file_not_relative_to_base_dir" );
            KURL old = doc->url();
            KURL newOne = m_projectBaseURL;
            newOne.addPath( old.fileName() );
            bool ok = false;
            int i = 1;
            while (!ok)
            {
                if ( QFile::exists( newOne.path() ) )
                {
                    newOne = m_projectBaseURL;
                    newOne.addPath( QString("copy_%1_%2").arg(i).arg(old.fileName()) );
                }
                else
                    ok = true;
                i++;
            }
            doc->setUrl( newOne );
            doc->save();
            doc->kateDoc()->setModified( false );
        }
        KLProjectManagerWidget* it;
        for (it=m_projectManagerWidgets.first(); it; it=m_projectManagerWidgets.next())
        {
            it->addDocument( doc );
        }
        m_parent->beAwareOfChangesOf( doc );
    }
    else
        qWarning( "Tried to append a document pointer twice." );

    // recreateListView();
}


void KLProject::removeDocument( KLDocument * doc )
{
    if ( m_documents.findRef( doc ) >= 0 )
    {
        m_documents.removeRef( doc );
        m_unsaved=true;
        update();
    }
}


QPtrList< KLDocument > KLProject::getAllDocsOfType( const KLDocumentType & type )
{
    QPtrList< KLDocument > retVal;
    retVal.setAutoDelete( false );
    KLDocument* it;
    for ( it = m_documents.first(); it; it = m_documents.next() )
    {
        if ( it->type() == type )
            retVal.append( it );
    }
    return retVal;
}


void KLProject::recreateListView( )
{
    KLProjectManagerWidget* it;
    for (it=m_projectManagerWidgets.first(); it; it=m_projectManagerWidgets.next())
    {
        it->update();
    }
}


void KLProject::recreateListItemFor( KLDocument * doc )
{
    KLProjectManagerWidget* it;
    for (it=m_projectManagerWidgets.first(); it; it=m_projectManagerWidgets.next())
    {
        it->update( doc );
    }
}


void KLProject::compile( )
{
    if ( activeDocument() )
    {
        m_parent->clearMessages();
        m_parent->procManager()->setFinishMessage( i18n("File compiled successfully.") );
        m_compilerInterface->compile( activeDocument() );
    }
}


void KLProject::erase()
{
    m_programmerInterface->configuration() = m_parent->programmerConfig();
    m_parent->procManager()->setFinishMessage( i18n("Chip successfully erased.") );
    m_programmerInterface->erase();
}


bool KLProject::upload( const KURL & url )
{
    bool retVal = false;
    m_programmerInterface->configuration() = m_parent->programmerConfig();
    QString selCpu = KInputDialog::getItem( i18n("Select CPU"),
        i18n("Please select the type of CPU to upload to."), m_cpus, cpuIndex(), false );
    if ( (!selCpu.isNull()) && (!selCpu.isEmpty()) )
    {
        QString oldCpu = cpu();
        setCpu( selCpu );
        m_parent->procManager()->setFinishMessage( i18n("File uploaded successfully.") );
        retVal = m_programmerInterface->upload( url );
        setCpu( oldCpu );
    }
    return retVal;
}


bool KLProject::verify()
{
    m_programmerInterface->configuration() = m_parent->programmerConfig();
    m_parent->procManager()->setFinishMessage( i18n("Project verified successfully.") );
    return m_programmerInterface->verify( getHEXURL() );
}


bool KLProject::download( const KURL & url )
{
    bool retVal = false;
    m_programmerInterface->configuration() = m_parent->programmerConfig();
    QString selCpu = KInputDialog::getItem( i18n("Select CPU"),
        i18n("Please select the type of CPU to download from."), m_cpus, cpuIndex(), false );
    if ( (!selCpu.isNull()) && (!selCpu.isEmpty()) )
    {
        QString oldCpu = cpu();
        setCpu( selCpu );
        m_parent->procManager()->setFinishMessage( i18n("File downloaded successfully.") );
        retVal = m_programmerInterface->download( url );
        setCpu( oldCpu );
    }
    return retVal;
}


bool KLProject::ignite()
{
    if ( m_projectFileURL.isValid() )
        save( m_projectFileURL );
    m_programmerInterface->configuration() = m_parent->programmerConfig();
    buildAll();
    m_parent->procManager()->setFinishMessage( i18n("Project built and uploaded successfully.") );
    m_programmerInterface->ignite( getHEXURL() );
    return true;
}


bool KLProject::programFuses( QMap< QString, QString > fuses, const QString mmcu )
{
    bool retVal = false;
    m_programmerInterface->configuration() = m_parent->programmerConfig();
    QString selCpu = mmcu;
    if ( selCpu.isEmpty() || selCpu.isNull() )
        selCpu = KInputDialog::getItem( i18n("Select CPU"),
                                        i18n("Please select the type of CPU to write the fuses."), m_cpus, cpuIndex(), false );
    if ( (!selCpu.isNull()) && (!selCpu.isEmpty()) )
    {
        QString oldCpu = cpu();
        setCpu( selCpu );
        m_parent->procManager()->setFinishMessage( i18n("Successfully programmed the fuse bits.") );
        retVal = m_programmerInterface->programFuses( fuses );
        setCpu( oldCpu );
    }

    return retVal;
}


bool KLProject::readFuses( const QString mmcu )
{
    bool retVal = false;
    m_programmerInterface->configuration() = m_parent->programmerConfig();
    QString selCpu = mmcu;
    if ( selCpu.isEmpty() || selCpu.isNull() )
        selCpu = KInputDialog::getItem( i18n("Select CPU"),
                                        i18n("Please select the type of CPU to read the fuses from."), m_cpus, cpuIndex(), false );
    if ( (!selCpu.isNull()) && (!selCpu.isEmpty()) )
    {
        QString oldCpu = cpu();
        setCpu( selCpu );
        m_parent->procManager()->setFinishMessage( i18n("Successfully read the fuse bits.") );
        retVal = m_programmerInterface->readFuses();
        setCpu( oldCpu );
    }

    return retVal;
}


bool KLProject::upload( )
{
    m_parent->procManager()->setFinishMessage( i18n("File successfully uploaded.") );
    m_programmerInterface->configuration() = m_parent->programmerConfig();
    return m_programmerInterface->upload( getHEXURL() );
}

bool KLProject::hexFileIsUpToDate( )
{
    buildAll();
    return true;
}


void KLProject::registerProjectManager( KLProjectManagerWidget * prMan )
{
    if ( m_projectManagerWidgets.findRef( prMan ) < 0 )
        m_projectManagerWidgets.append( prMan );
    else
        qWarning(i18n("Tried to append a project manager twice: %s.").ascii(), prMan->name());
}


void KLProject::unregisterProjectManager( KLProjectManagerWidget * prMan )
{
    if ( m_projectManagerWidgets.findRef( prMan ) >= 0 )
        m_projectManagerWidgets.remove( prMan );
    else
        qWarning(i18n("Tried to remove a project manager twice: %s"), prMan->name());
}

KLCPUFeatures KLProject::cpuFeaturesFor( const QString & cpu )
{
    QValueList<KLCPUFeatures>::iterator it;
    for (it=m_cpuFeatures.begin(); it!=m_cpuFeatures.end(); ++it)
        if ((*it).name()==cpu)
            return (*it);
    return KLCPUFeatures();
}

void KLProject::initialize( const KURL& url )
{
    if ( url.isEmpty() || (!url.isValid()) )
        m_projectBaseURL = KFileDialog::getExistingDirectory( ".", m_parent, i18n( "Select project base directory" ) );
    else
        m_projectBaseURL = url;

    if ( m_projectBaseURL.isValid() &&
         (!m_projectBaseURL.isEmpty()) && 
         m_projectBaseURL.isLocalFile() )
    {
        m_initialized = true;
        m_documents.clear();
        KLProjectManagerWidget* it;
        m_name = m_projectBaseURL.fileName();
        for (it=m_projectManagerWidgets.first(); it; it=m_projectManagerWidgets.next())
        {
            it->setProjectName( m_projectBaseURL.fileName() );
        }
    }
    else
        m_initialized = false;
    m_unsaved=true;
}


QString KLProject::attribute( const QString & attr, const QString & confKey, bool ifTrueAttribOnly ) const
{
    QString retVal;
    if ( m_settings.contains( confKey ) )
    {
        retVal = m_settings[ confKey ];
        if ( (!retVal.isNull()) && (!retVal.isEmpty()) )
        {
            if ( ifTrueAttribOnly && (retVal == TRUE_STRING) )
                return " " + attr;
            else if (!ifTrueAttribOnly)
                return " " + attr + retVal;
        }
    }
    return "";
}


QString KLProject::attributeLinkerSection( const QString & attr,
                                           const QString & confKey ) const
{
    QString retVal;
    int intRetVal;
    bool ok;

    if ( m_settings.contains( confKey ) )
    {
        retVal = m_settings[ confKey ];
        intRetVal = retVal.toInt( &ok, 16 );
        if ( (!retVal.isNull()) && (!retVal.isEmpty()) && ok )
        {
            return " " + attr + retVal.setNum( (ulong) (0x800000 + intRetVal) , 16 );
        }
    }
    return "";
}


QString KLProject::getObjectFileNameFor( const KURL & file )
{
    QString oFileName = file.fileName();
    oFileName = oFileName.left( oFileName.find( "." ) ) + ".o";
    return oFileName;
}

QString KLProject::conf( const QString & confKey, const QString & defval ) const
{
    if ( m_settings.find( confKey ) != m_settings.end() )
    {
        return m_settings[ confKey ];
    }
    else
    {
        kapp->config()->setGroup("KontrollerLab");
        QString val = kapp->config()->readEntry( confKey, "" );
        // qDebug("%s", val.ascii() );
        if ( (!val.isEmpty()) && (!val.isNull()) )
            return val;
        else
            return defval;
    }
}

QString KLProject::getOUTFileName( )
{
    QString buf = conf( PRJ_HEX_FILE, "project.hex" );
    buf = buf.left( buf.find( "." ) ) + ".out";
    return buf;
}

void KLProject::createDOMElement( QDomDocument & document, QDomElement & parent )
{
    QDomElement settings = document.createElement( "SETTINGS" );
    fromQMapToDOM( document, settings, m_settings );
    QDomElement programmerConfig = document.createElement( "PROGRAMMERCONFIG" );
    fromQMapToDOM( document, programmerConfig, m_parent->programmerConfig() );
    
    QDomElement files = document.createElement( "FILES" );
    KLDocument* it;
    for (it=m_documents.first(); it; it=m_documents.next())
    {
        QDomElement curfile = document.createElement( "FILE" );
        curfile.setAttribute( "NAME", it->name() );
        curfile.setAttribute( "SHOWN", it->registeredViewsCount()>0 ? TRUE_STRING : FALSE_STRING );
        files.appendChild( curfile );
    }
    
    parent.appendChild( files );
    parent.appendChild( settings );
    parent.appendChild( programmerConfig );
    
}

void KLProject::fromQMapToDOM( QDomDocument & document, QDomElement & parent,
                               QMap<QString, QString> map )
{
    QMap<QString, QString>::Iterator it;
    for (it=map.begin(); it!=map.end(); ++it)
    {
        QDomElement elem = document.createElement( it.key().upper() );
        elem.setAttribute( "VALUE", it.data() );
        parent.appendChild( elem );
    }
}

QMap< QString, QString > KLProject::fromDOMToQMap( QDomDocument &, QDomElement & element )
{
    QMap< QString, QString > retVal;

    QDomNode n = element.firstChild();
    while( !n.isNull() )
    {
        QDomElement e = n.toElement(); // try to convert the node to an element.
        if( !e.isNull() )
        {
            retVal[ e.tagName().upper() ] = e.attribute( "VALUE", "" );
            // qDebug("retVal[ %s ] = %s", e.tagName().upper().ascii(), retVal[ e.tagName().upper() ].ascii() );
        }
        n = n.nextSibling();
    }
    return retVal;
}

void KLProject::save( const KURL & url )
{
    if ( url.isLocalFile() )
    {
        m_projectFileURL = url;
        m_name = m_projectBaseURL.fileName();
        QDomDocument doc("KontrollerLab");
        QDomElement root = doc.createElement( "PROJECT" );
        root.setAttribute( "VERSION", VERSION );
        doc.appendChild( root );

        createDOMElement( doc, root );

        QFileInfo finfo( url.path() );
        QFile file( url.path() );

        if ( finfo.exists() )
            file.remove();
        file.open( IO_ReadWrite );
        QString xml = doc.toString( 2 );
        QByteArray ba;
        ba.setRawData( xml.ascii(), xml.length() );
        QTextStream stream( &file );
        stream << xml;
        ba.resetRawData( xml.ascii(), xml.length() );
        file.close();
        saveAllDocs();
        m_unsaved=false;
        checkForModifiedFiles();
        update();
    }
}


void KLProject::open( const KURL & url )
{
    QString localfilename;
    QString fname = url.fileName();
    
    if (url.isLocalFile())
        localfilename = url.path();
    else if (!KIO::NetAccess::download(url, localfilename, (QWidget*)parent()))
    {
        qWarning( "Download failed: %s", fname.ascii() );
        return;
    }
    
    QFile qfile (localfilename);
    if (qfile.open(IO_ReadOnly))
    {
        QDomDocument doc("KontrollerLab");

        initialize( url.directory() );
        m_projectFileURL = url;

        QString errorMesg;
        int errorLine, errorCol;
        if ( !doc.setContent( &qfile, false, &errorMesg, &errorLine, &errorCol ) ) {
            qWarning( "Error in file %s: %d(%d) %s", fname.ascii(), errorLine, errorCol, errorMesg.ascii() );
            qfile.close();
            return;
        }
        qfile.close();

        QDomElement docElem = doc.documentElement();
        readFromDOMElement( doc, docElem );
        m_unsaved=false;
        updateHexFileSize();
    }
    else
        qWarning("Could not open file.");
}


void KLProject::readFromDOMElement( QDomDocument & doc, QDomElement & element )
{
    QDomNode n = element.firstChild();
    while( !n.isNull() ) {
        QDomElement e = n.toElement(); // try to convert the node to an element.
        if( !e.isNull() )
        {
            if ( e.tagName().lower() == "settings" )
            {
                m_settings = fromDOMToQMap( doc, e );
                // qDebug( "m_settings[CPU]=%s", m_settings[PRJ_CPU].ascii() );
            }
            else if ( e.tagName().lower() == "programmerconfig" )
            {
                m_parent->setProgrammerConfig( fromDOMToQMap( doc, e ) );
            }
            else if ( e.tagName().lower() == "files" )
            {
                QStringList fileShown;
                KLDocumentView* firstview=0L;
                QStringList files = fromDOMToQStringList( doc, e, "FILE", "SHOWN", TRUE_STRING, fileShown );
                QStringList::iterator it, showIt;
                showIt = fileShown.begin();
                for (it = files.begin(); it != files.end(); ++it)
                {
                    if ( (!(*it).isEmpty()) && (!(*it).isNull()) )
                    {
                        // If the init failed, return!
                        if (!initialized())
                            break;
    
                        KLDocument* docu = new KLDocument(m_parent);
                        KURL openURL = m_projectBaseURL;
                        openURL.addPath( *it );
                        docu->open( openURL );
                        addDocument( docu );
                        if ( (*showIt).upper() == TRUE_STRING )
                        {
                            KLDocumentView* view = new KLDocumentView( docu, m_parent );
                            if (!firstview)
                                firstview = view;
                            view->show();
                        }
                    }
                    ++showIt;
                }
                // To avoid multiple toolbar entries (occurred on older KDE's):
                if (firstview)
                    firstview->show();
            }
        }
        n = n.nextSibling();
    }
}

QStringList KLProject::fromDOMToQStringList( QDomDocument &, QDomElement & element, const QString & tag,
                                             const QString & attribName, const QString & attribDefValue,
                                             QStringList &attribs )
{
    QStringList retVal;

    QDomNode n = element.firstChild();
    while( !n.isNull() )
    {
        QDomElement e = n.toElement(); // try to convert the node to an element.
        if( !e.isNull() )
        {
            if ( e.tagName().upper() == tag.upper() )
            {
                retVal.append( e.attribute( "NAME", "" ) );
                attribs.append( e.attribute( attribName, attribDefValue ) );
            }
        }
        n = n.nextSibling();
    }
    return retVal;
}

void KLProject::buildAll( )
{
    m_parent->clearMessages();
    m_parent->procManager()->setFinishMessage( i18n("Project built successfully.") );
    m_compilerInterface->buildAll();
}

void KLProject::rebuildAll( )
{
    m_parent->clearMessages();
    m_parent->procManager()->setFinishMessage( i18n("Project built successfully.") );
    m_compilerInterface->rebuildAll();
}

void KLProject::close( )
{
    m_initialized = false;
    m_projectBaseURL = KURL();
    m_projectFileURL = KURL();
    m_name = i18n("Unnamed project");
    m_unsaved = false;
    m_parent->setCaption( "KontrollerLab" );

    KLProjectManagerWidget* it;
    for (it=m_projectManagerWidgets.first(); it; it=m_projectManagerWidgets.next())
    {
        it->clear();
        it->update();
    }

    QPtrList< KLDocument > list = m_documents;
    for ( KLDocument* it = list.first(); it; it = list.next() )
        removeDocument( it );
}

int KLProject::cpuIndex( ) const
{
    return m_cpus.findIndex( cpu() );
}

int KLProject::getHexFileSize( )
{
    QString buffer;
    int retVal = 0;
    QFile hexFile( getHEXURL().path() );
    
    if ( hexFile.open( IO_ReadOnly ) )
    {
        while ( hexFile.readLine( buffer, 32768 ) >= 0 )
        {
            if ( buffer.length() < 9 )
                continue;
            if ( buffer.mid( 0, 1 ) != ":" )
                continue;
            QString bCount_ = buffer.mid( 1, 2 );
            QString type_ = buffer.mid( 7, 2 );
            bool ok1, ok2;
            int bCount = bCount_.toInt( &ok1, 16 );
            int type = type_.toInt( &ok2, 16 );
            if ( ok1 && ok2 && (type==0) )
                retVal += bCount;
        }
    }
    return retVal;
}

void KLProject::updateHexFileSize( )
{
    int size = getHexFileSize();

    if ( size > 0 )
    {
        KLProjectManagerWidget* it;
        for (it=m_projectManagerWidgets.first(); it; it=m_projectManagerWidgets.next())
        {
            it->setHexFileSize( size );
        }
    }
}


void KLProject::checkForModifiedFiles( )
{
    for ( KLProjectManagerWidget* itPM = m_projectManagerWidgets.first();
          itPM; itPM = m_projectManagerWidgets.next() )
    {
        itPM->updateModified();
    }
    QPtrList<KLDocument> copyList = m_documents;
    copyList.setAutoDelete( false );

    for ( KLDocument* it = copyList.first(); it; it = copyList.next() )
        it->updateModified();
}


KLCPUFuses KLProject::getFusesFor( const QString & name )
{
    KLCPUFuses retVal;
    QValueList< KLCPUFuses >::iterator it;
    for ( it = m_cpuFuses.begin(); it != m_cpuFuses.end(); ++it )
    {
        if ( (*it).mcuName().upper() == name.upper() )
        {
            retVal = (*it);
            break;
        }
    }
    return retVal;
}


void KLProject::backannotateFuses( QMap< QString, QString > fuses )
{
    m_parent->backannotateFuses( fuses );
}

// This is ONLY called when an error occurred:
void KLProject::backannotateFuses( )
{
    m_parent->slotMessageBox( 1, i18n("The fuse bits could not be read correctly.") );
}

bool KLProject::unsaved( )
{
    if (m_unsaved)
    {
        return true;
    }
    KLDocument* it;
    QPtrList<KLDocument> copyList = m_documents;
    copyList.setAutoDelete( false );
    
    for (it=copyList.first(); it; it=copyList.next())
    {
        if (it->isModified())
        {
            return true;
        }
    }
    return false;
}

void KLProject::saveAllDocs( )
{
    KLDocument* it;
    QPtrList<KLDocument> copyList = m_documents;
    copyList.setAutoDelete( false );

    for (it=copyList.first();it;it=copyList.next())
        it->save();
}


void KLProject::backannotateSTK500( const QString& stdout )
{
    m_parent->backannotateSTK500( stdout );
}


/**
 *
 * @param name The name of the desired document.
 * @return a pointer to the document, or 0L is there is no such document.
 */
KLDocument * KLProject::getDocumentNamed( const QString & name )
{
    QPtrList< KLDocument > list = m_documents;
    for ( KLDocument* it = list.first(); it; it = list.next() )
    {
        if ( it->name() == name )
        {
            return it;
        }
    }
    return 0L;
}


void KLProject::setProgrammerBusy( bool val )
{
    m_programmerBusy = val;
    m_parent->setProgrammerBusy( val );
}

